<?php
// no direct access
defined('ABSPATH') or die();

$api = LSD_Options::api();
?>
<div class="lsd-settings-wrap">
    <form id="lsd_api_form">
        <div class="lsd-settings-form-group">
            <h3><?php esc_html_e('API Tokens', 'listdom'); ?></h3>
            <div class="lsd-mt-4 lsd-mb-5">
                <p class="description"><?php esc_html_e("Do not remove a token if an application is using it because it will destroy functionality of the application! You can insert a descriptive name for any token.", 'listdom'); ?></p>
                <button type="button" class="button" id="lsd_settings_api_add_token"><?php esc_html_e('Add Token', 'listdom'); ?></button>
            </div>
            <?php foreach($api['tokens'] as $i => $token): ?>
            <div class="lsd-form-row" id="lsd_settings_api_tokens_<?php echo esc_attr($i); ?>">
                <div class="lsd-col-2"><?php echo LSD_Form::text(array(
                    'id' => 'lsd_settings_api_tokens_'.esc_attr($i).'_name',
                    'name' => 'lsd[tokens]['.esc_attr($i).'][name]',
                    'value' => $token['name'],
                    'placeholder' => esc_html__('Token Name', 'listdom'),
                )); ?></div>
                <div class="lsd-col-4">
                    <input type="text" name="lsd[tokens][<?php echo esc_attr($i); ?>][key]" id="lsd_settings_api_tokens_<?php echo esc_attr($i); ?>_key"  value="<?php echo esc_attr($token['key']); ?>" placeholder="<?php esc_attr_e('Token Key', 'listdom'); ?>" readonly>
                </div>
                <div class="lsd-col-1">
                    <div class="lsd-api-remove-token lsd-pt-2 lsd-cursor-pointer" data-i="<?php echo esc_attr($i); ?>" data-confirm="0"><i class="lsd-icon fas fa-trash-alt"></i></div>
                </div>
            </div>
            <?php endforeach; ?>
        </div>
        <div class="lsd-spacer-10"></div>
        <div class="lsd-form-row">
			<div class="lsd-col-12">
				<?php LSD_Form::nonce('lsd_api_form'); ?>
				<?php echo LSD_Form::submit(array(
					'label' => esc_html__('Save', 'listdom'),
					'id' => 'lsd_api_save_button',
					'class' => 'button button-hero button-primary',
				)); ?>
			</div>
        </div>
    </form>
</div>
<script>
// Add Token
jQuery('#lsd_settings_api_add_token').on('click', function()
{
    var $button = jQuery("#lsd_settings_api_add_token");

    // Add loading Class to the button
    $button.addClass('loading').html('<i class="lsd-icon fa fa-spinner fa-pulse fa-fw"></i>');

    jQuery.ajax(
    {
        type: "POST",
        url: ajaxurl,
        data: "action=lsd_api_add_token&_wpnonce=<?php echo wp_create_nonce('lsd_api_add_token'); ?>",
        dataType: "json",
        success: function(response)
        {
            // Remove loading Class from the button
            $button.removeClass('loading').html("<?php echo esc_js(esc_attr__('Add Token', 'listdom')); ?>");

            if(response.success === 1) location.reload();
        },
        error: function()
        {
            // Remove loading Class from the button
            $button.removeClass('loading').html("<?php echo esc_js(esc_attr__('Add Token', 'listdom')); ?>");
        }
    });
});

// Remove Token
jQuery('.lsd-api-remove-token').on('click', function()
{
    var $button = jQuery(this);
    var i = $button.data('i');

    var confirm = $button.data('confirm');
    if(!confirm)
    {
        $button.data('confirm', 1);
        $button.addClass('lsd-need-confirm');

        setTimeout(function()
        {
            $button.data('confirm', 0);
            $button.removeClass('lsd-need-confirm');
        }, 5000);

        return false;
    }

    // Add loading Class to the button
    $button.removeClass('lsd-need-confirm').addClass('loading').html('<i class="lsd-icon fa fa-spinner fa-pulse fa-fw"></i>');

    jQuery.ajax(
    {
        type: "POST",
        url: ajaxurl,
        data: "action=lsd_api_remove_token&_wpnonce=<?php echo wp_create_nonce('lsd_api_remove_token'); ?>&i="+i,
        dataType: "json",
        success: function(response)
        {
            if(response.success === 1)
            {
                // Remove Token
                jQuery('#lsd_settings_api_tokens_'+i).remove();
            }
        },
        error: function()
        {
            // Remove loading Class from the button
            $button.removeClass('loading').html("<?php echo esc_js(esc_attr__('Remove', 'listdom')); ?>");
        }
    });
});

// Save
jQuery('#lsd_api_form').on('submit', function(event)
{
    event.preventDefault();

    // Add loading Class to the button
    var $button = jQuery("#lsd_api_save_button");
    $button.addClass('loading').html('<i class="lsd-icon fa fa-spinner fa-pulse fa-fw"></i>');

    var settings = jQuery("#lsd_api_form").serialize();
    jQuery.ajax(
    {
        type: "POST",
        url: ajaxurl,
        data: "action=lsd_save_api&" + settings,
        success: function()
        {
            // Remove loading Class from the button
            $button.removeClass('loading').html("<?php echo esc_js(esc_attr__('Save', 'listdom')); ?>");
        },
        error: function()
        {
            // Remove loading Class from the button
            $button.removeClass('loading').html("<?php echo esc_js(esc_attr__('Save', 'listdom')); ?>");
        }
    });
});
</script>